#!/bin/sh

# Note:
#  Amongst others this script updates the MCU firmware (ControlPanel) of
#  the AlarmSystem.
#  The check for the need to update the MCU or not is said to be implemented in
#  the updater binary (maybe, someday).
#  Therefor the whole functionality for this is found in here, though other
#  devices handle similar actions different.
#
#  In the not so rare cases the firmware could not be installed this script
#  enters an infinite loop. Under normal cicumstances this leads to a watchdog reset.
#  It sometimes happens that, for whatever reason, the ipmodule is reset by the
#  update process. The hereafter following attempt WILL fail as the updater is not
#  able to read the 'old' firmware version and returns an exit code 2.
#  It is unknown how harmfull this is or might become and thus the update is considered
#  as failure.

me="[alarmsystem-startup.sh]"

MCU_UPDATER_CMD="/opt/gira/bin/CrowMcuUpdateFW"
MCU_PERIPHERAL_UPDATER_CMD="/opt/gira/bin/CrowMcuPeripheralUpdate"

CROW_FIRMWARE_DIR="/opt/gira/mcu"

MCU_FIRMWARE_PATH=$( readlink -f ${CROW_FIRMWARE_DIR}/ControlPanel.bin )
RF_MODULE_FIRMWARE_PATH=$( readlink -f ${CROW_FIRMWARE_DIR}/RFModule.bin )
MCU_LANGUAGE_RESSOURCE_PATH=$( readlink -f ${CROW_FIRMWARE_DIR}/Languages.xml )
MCU_VOICE_RESSOURCE_PATH=$( readlink -f ${CROW_FIRMWARE_DIR}/Voice.mem )

HASH_OF_INSTALLED_MCU_FIRMWARE="/opt/extparam/actual_mcu_firmware.bin.md5"
HASH_OF_INSTALLED_RF_MODULE_FIRMWARE="/opt/extparam/actual_rf_module_firmware.bin.md5"
HASH_OF_INSTALLED_MCU_LANGUAGE_RESSOURCE="/opt/extparam/actual_mcu_language_ressource.xml.md5"
HASH_OF_INSTALLED_MCU_VOICE_RESSOURCE="/opt/extparam/actual_mcu_voice_ressource.mem.md5"


ALARMSYSTEMAPP_RUN="/opt/gira/bin/alarmsystemapp --configFile=/opt/gira/etc/alarmsystemapp/AlarmSystemAppConfig.xml"
IPMAPP_RUN="/opt/gira/bin/IPMApp.sh"

LOG_FILE=/opt/userdata/CrowUpdate.log

# PIDs of forked child processes used in order to cleanup in trap function.
PIDS=""

printLog()
{
#  echo "$(date -u +%Y-%m-%dT%H:%M:%S)+00:00 $*"
  echo "$(date -u '+%Y-%m-%d %H:%M:%S') $*" >> ${LOG_FILE}
}

moveIfExists()
{
  if [ $# -lt 2 ]
  then
    exit
  fi

  if [ -e ${1} ]
  then
    mv ${1} ${2}
  fi
}

rotateLog()
{
  for newPostfix in $(seq 5 -1 1)
  do
    if [ ${newPostfix} -gt 1 ]
    then
      oldPostfix=$(dc "${newPostfix} 1 - p")
      moveIfExists ${LOG_FILE}.${oldPostfix} ${LOG_FILE}.${newPostfix}
    else
      moveIfExists ${LOG_FILE} ${LOG_FILE}.${newPostfix}
    fi
  done
}

trapFunction()
{
  trap - SIGHUP SIGINT SIGTERM

  printLog "Will shutdown! Please wait ..."
  kill $PIDS
  killall IPMApp

  exit 2
}

waitForWatchdog()
{
  printLog "Waiting for the watchdog to bite."
  while /bin/true
  do
    sleep 600
    printLog "Still waiting for the watchdog to bite."
  done
}

crowMcuUpdate()
{
  local MCU_UPDATER_CMD_EXIT_CODE

  printLog "Starting Crow MCU firmware update."

  rm -f ${HASH_OF_INSTALLED_MCU_FIRMWARE}
  ${MCU_UPDATER_CMD} -bl ${MCU_FIRMWARE_PATH} >> ${LOG_FILE} 2>&1
  local updateResult=$?
  printLog "Crow MCU firmware update finished with exit code ${updateResult}."
  if [ ${updateResult} -gt 3 ]
  then
    if [ ${updateResult} -eq 127 ]
    then
      printLog "${MCU_UPDATER_CMD} couldn't be started!"
    fi

    printLog "Crow MCU update failed with exit code ${updateResult}."
    printLog "Firmware ${MCU_FIRMWARE_PATH} could not be installed."
    waitForWatchdog
  fi

  md5sum ${MCU_FIRMWARE_PATH} > ${HASH_OF_INSTALLED_MCU_FIRMWARE}
  printLog "Firmware ${MCU_FIRMWARE_PATH} successfully installed."
}

# $1 = Type of update (rfm, language or voice).
# $2 = File path of the firmware.
# $3 = File path of the firmware hash file.
crowPeripheralUpdate()
{
  local updateType=$1
  local firmwarePath=$2
  local firmwareHashPath=$3

  if ! md5sum -s -c ${firmwareHashPath}
  then
    printLog "Starting Crow peripheral update of type ${updateType}."

    rm -f ${firmwareHashPath}
    ${MCU_PERIPHERAL_UPDATER_CMD} -update ${updateType} ${firmwarePath} >> ${LOG_FILE} 2>&1
    local updateResult=$?
    printLog "Crow peripheral update of type ${updateType} finished with exit code ${updateResult}."
    if [ ${updateResult} -gt 0 ]
    then
      if [ ${updateResult} -eq 127 ]
      then
        printLog "${MCU_PERIPHERAL_UPDATER_CMD} couldn't be started!"
      fi

      printLog "Crow peripheral update of type ${updateType} failed with exit code ${updateResult}."
      printLog "File ${firmwarePath} could not be installed."
      waitForWatchdog
    fi

    md5sum ${firmwarePath} > ${firmwareHashPath}
    printLog "File ${firmwarePath} successfully installed."
  else
    printLog "Hashes for the update type ${updateType} are equal. No need to update."
  fi
}

crowPeripheralGGUpdate()
{
  printLog "Starting Crow peripheral update of type gas gauge."

  ${MCU_PERIPHERAL_UPDATER_CMD} -update gg >> ${LOG_FILE} 2>&1
  local updateResult=$?
  printLog "Crow peripheral update of type gas gauge finished with exit code ${updateResult}."
  if [ ${updateResult} -gt 0 ]
  then
    # In order to trigger the gas gauge update again at the run, we have to remove the MCU hash here.
    rm -f ${HASH_OF_INSTALLED_MCU_FIRMWARE}

    if [ ${updateResult} -eq 127 ]
    then
      printLog "${MCU_PERIPHERAL_UPDATER_CMD} couldn't be started!"
    fi

    printLog "Crow peripheral update of type gas gauge failed with exit code ${updateResult}."
    waitForWatchdog
  fi

  printLog "Gas gauge successfully installed."
}




rotateLog
printLog "Start of script execution."

# Update of Crow MCU firmware.
if ! md5sum -s -c ${HASH_OF_INSTALLED_MCU_FIRMWARE}
then
  crowMcuUpdate
  MCU_UPDATED=true
else
  printLog "Matching Crow MCU firmware hash found. No need to update."
  MCU_UPDATED=false
fi

# Start of Crow IPMApp.
printLog "Starting $IPMAPP_RUN"
# Just start IPMApp with parameter, if developer only device package has been installed.
if [ -e /opt/userdata/.developer-only-installed ]
then
  IPM_APP_PARAMETERS="--bind-external-ports"
fi
$IPMAPP_RUN ${IPM_APP_PARAMETERS} 2>&1 &
PIDS="$PIDS $!"

# Update Crow gas gauge.
if [ ${MCU_UPDATED} = "true" ]
then
  crowPeripheralGGUpdate
fi

# Update of Crow RF-Module, language resources and voice resources.
crowPeripheralUpdate "rfm"   ${RF_MODULE_FIRMWARE_PATH}     ${HASH_OF_INSTALLED_RF_MODULE_FIRMWARE}
crowPeripheralUpdate "lang"  ${MCU_LANGUAGE_RESSOURCE_PATH} ${HASH_OF_INSTALLED_MCU_LANGUAGE_RESSOURCE}
crowPeripheralUpdate "voice" ${MCU_VOICE_RESSOURCE_PATH}    ${HASH_OF_INSTALLED_MCU_VOICE_RESSOURCE}

# Start of alarm system application.
printLog "Starting $ALARMSYSTEMAPP_RUN"
$ALARMSYSTEMAPP_RUN 2>&1 &
PIDS="$PIDS $!"

trap trapFunction SIGHUP SIGINT SIGTERM

printLog "Waiting for processes to exit. PIDs =${PIDS}."
wait $PIDS

exit 0
